#include <glib.h>
#include <libxml/tree.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "preferences.h"

static void kf_preferences_dir (void);
static void kf_preferences_dump (void);
static void kf_preferences_load_accounts (xmlDocPtr doc, xmlNodePtr node);
static void kf_preferences_write_accounts (FILE *f);
static void kf_preferences_load_statuses (xmlDocPtr doc, xmlNodePtr node);
static void kf_preferences_write_statuses (FILE *f);
static GSList *kf_preferences_load_list (xmlDocPtr doc, xmlNodePtr node, GSList *list);

typedef enum {
	KF_PREF_TYPE_INT,
	KF_PREF_TYPE_STRING
} KfPrefType;

typedef struct {
	gchar *name;
	KfPrefType type;
	gint value_int;
	gchar *value_string;
} KfPref;

/*
 * Ustawienia programu
 */
static KfPref kf_preferences[] = {
	{"autoPopup",	KF_PREF_TYPE_INT,	1, NULL},
	{"autoAwayEnabled",	KF_PREF_TYPE_INT,	1, NULL},
	{"autoAwayDelay",	KF_PREF_TYPE_INT,	5, NULL},
	{"autoXADelay",	KF_PREF_TYPE_INT,	15, NULL},
	{"autoAwayText",KF_PREF_TYPE_STRING,	0, "AutoAway"},
	{"autoXAText",	KF_PREF_TYPE_STRING,	0, "AutoXA"},
	{"autoPopup",	KF_PREF_TYPE_INT,	1, NULL},
	{"colorChatHe",	KF_PREF_TYPE_STRING,	0, "#A52A2A"},
	{"colorChatMe",	KF_PREF_TYPE_STRING,	0, "#1E90FF"},
	{"colorChatUrl",KF_PREF_TYPE_STRING,	0, "blue"},
	{"browser",	KF_PREF_TYPE_STRING,	0, "mozilla"},
	{"statusType",	KF_PREF_TYPE_STRING,	0, NULL},
	{"statusTypeInt",	KF_PREF_TYPE_INT,	0, NULL},
	{"statusText",	KF_PREF_TYPE_STRING,	0, NULL},
	{"showOffline",	KF_PREF_TYPE_INT,	0, NULL},
	{"showToolbar",	KF_PREF_TYPE_INT,	1, NULL},
	{"colourRows",	KF_PREF_TYPE_INT,	0, NULL},
	{"messageDefault",	KF_PREF_TYPE_INT,	0, NULL},
	{"posX",	KF_PREF_TYPE_INT,	0, NULL},
	{"posY",	KF_PREF_TYPE_INT,	0, NULL},
	{"sizeX",	KF_PREF_TYPE_INT,	0, NULL},
	{"sizeY",	KF_PREF_TYPE_INT,	0, NULL},
	{"soundEnabled",KF_PREF_TYPE_INT,	1, NULL},
	{"soundPlayer",	KF_PREF_TYPE_STRING,	0, "play"},
	{"tabDefault",	KF_PREF_TYPE_INT,	0, NULL},
	{"useEmoticons",	KF_PREF_TYPE_INT,	1, NULL},
	{NULL, 0, 0, NULL}
};

GList *kf_preferences_accounts = NULL;
GList *kf_preferences_statuses = NULL;

GSList *kf_preferences_blocked_jids = NULL;

/*
 * aduje preferencje z pliku
 */
void kf_preferences_load (const gchar *filename) {
	xmlDocPtr doc;
	xmlNodePtr node;

	doc = xmlParseFile (filename);
	if (doc == NULL) {
//		g_error ("Nie ma pliku config.xml!\n");
		xmlFreeDoc (doc);
		return;
	}

	node = xmlDocGetRootElement (doc);
	if (node == NULL) {
//		g_error ("Nie ma gwnego elementu!\n");
		xmlFreeDoc (doc);
		return;
	}
	if (xmlStrcmp (node->name, (const xmlChar *) "configuration")) {
//		g_error ("Bdny format pliku konfiguracyjnego!\n");
		xmlFreeDoc (doc);
		return;
	}

	node = node->xmlChildrenNode;
	while (node) {
		if (node->type != XML_ELEMENT_NODE) {
			node = node->next;
			continue;
		}
		if (!xmlStrcmp (node->name, (const xmlChar *) "option")) {
			xmlChar *key, *value;
			key = xmlGetProp (node, "name");
//			value = xmlGetProp (node, "value");
			value = xmlNodeGetContent (node);
			if (key && value) {
				kf_preferences_set ((const gchar *) key, (const gchar *) value);
				xmlFree (key);
				xmlFree (value);
		}
		} else if (!xmlStrcmp (node->name, (const xmlChar *) "accounts")) {
			kf_preferences_load_accounts (doc, node->xmlChildrenNode);
		} else if (!xmlStrcmp (node->name, (const xmlChar *) "statuses")) {
			kf_preferences_load_statuses (doc, node->xmlChildrenNode);
		} else if (!xmlStrcmp (node->name, (const xmlChar *) "blacklist")) {
			 kf_preferences_blocked_jids = kf_preferences_load_list 
				 (doc, node->xmlChildrenNode, kf_preferences_blocked_jids);
		}
		node = node->next;
	}

//	kf_preferences_dump ();
	xmlFreeDoc (doc);
	xmlCleanupParser ();
}

/*
 * Ustawia zmienn konfiguracyjn name na warto value
 */
void kf_preferences_set (const gchar *name, const gchar *value) {
	KfPref *pref;

	for (pref = kf_preferences; pref && pref->name; pref++) {
		if (!strcmp (name, pref->name)) {
			if (pref->type == KF_PREF_TYPE_INT) {
				pref->value_int = atoi (value);
			} else if (pref->type == KF_PREF_TYPE_STRING) {
				pref->value_string = g_strdup (value);
			}
			break;
		}
	}
}

void kf_preferences_set_string (const gchar *name, gchar *value) {
	KfPref *pref;

	for (pref = kf_preferences; pref && pref->name; pref++) {
		if (!strcmp (name, pref->name)) {
			if (pref->type == KF_PREF_TYPE_INT) {
				pref->value_int = atoi (value);
			} else if (pref->type == KF_PREF_TYPE_STRING) {
				pref->value_string = value;
			}
			break;
		}
	}
}

void kf_preferences_set_int (const gchar *name, gint value) {
	KfPref *pref;

	for (pref = kf_preferences; pref && pref->name; pref++) {
		if (!strcmp (name, pref->name)) {
			if (pref->type == KF_PREF_TYPE_INT) {
				pref->value_int = value;
			} else if (pref->type == KF_PREF_TYPE_STRING) {
				pref->value_string = g_strdup_printf ("%d", value);
			}
			break;
		}
	}
}

gint kf_preferences_get_int (const gchar *name) {
	KfPref *pref;
//	gint value;

	for (pref = kf_preferences; pref && pref->name; pref++) {
		if (!strcmp (name, pref->name)) {
			if (pref->type == KF_PREF_TYPE_INT) {
				return pref->value_int;
			} else if (pref->type == KF_PREF_TYPE_STRING) {
//				pref->value_string = g_strdup (value);
//				return 0;
				return atoi (pref->value_string);
			}
			break;
		}
	}
}

const gchar *kf_preferences_get_string (const gchar *name) {
	KfPref *pref;
//	gchar *string;

	for (pref = kf_preferences; pref && pref->name; pref++) {
		if (!strcmp (name, pref->name)) {
			if (pref->type == KF_PREF_TYPE_INT) {
//				return g_strdup_printf ("%d", pref->value_int);
				return NULL;
			} else if (pref->type == KF_PREF_TYPE_STRING) {
				return pref->value_string;
			}
			break;
		}
	}
}

gpointer kf_preferences_get_ptr (const gchar *name) {
	KfPref *pref;
//	gchar *string;

	for (pref = kf_preferences; pref && pref->name; pref++) {
		if (!strcmp (name, pref->name)) {
			if (pref->type == KF_PREF_TYPE_INT) {
				return &(pref->value_int);
			} else if (pref->type == KF_PREF_TYPE_STRING) {
				return pref->value_string;
			}
			break;
		}
	}
}


/* Wypisuje warto wszystkich opcji - debug */
static void kf_preferences_dump (void) {
	KfPref *pref;

	for (pref = kf_preferences; pref && pref->name; pref++) {
		if (pref->type == KF_PREF_TYPE_INT) {
			g_print ("  <o n='%s' v='%d'/>\n", pref->name, pref->value_int);
		} else if (pref->type == KF_PREF_TYPE_STRING) {
			g_print ("  <o n='%s' v='%s'/>\n", pref->name, pref->value_string);
		}
	}

	if (1) {
		gchar **element;
		GList *list;
		KfPrefAccount *acc;
		list = kf_preferences_statuses;
		while (list) {
			element = list->data;
			g_print ("[[[ %s | %s ]]]\n", element[0], element[1]);
			list = list->next;
		}
	}
}

gboolean kf_preferences_write (const gchar *filename) {
	xmlDocPtr doc;
	xmlNodePtr root, node;
	KfPref *pref;
	/* Accounts */
	GList *tmp;
	GSList *tmps;
	KfPrefAccount *acc;
	
	doc = xmlNewDoc ("1.0");
	root = xmlNewNode (NULL, "configuration");
	xmlDocSetRootElement (doc, root);
	
	for (pref = kf_preferences; pref && pref->name; pref++) {
		if (pref->type == KF_PREF_TYPE_INT) {
			gchar *buffer;

			buffer = g_strdup_printf ("%d", pref->value_int);
			node = xmlNewTextChild (root, NULL, "option", buffer);
			g_free (buffer);
			xmlNewProp (node, "name", pref->name);
		} else if (pref->type == KF_PREF_TYPE_STRING && pref->value_string) {
			node = xmlNewTextChild (root, NULL, "option", pref->value_string);
			xmlNewProp (node, "name", pref->name);
		}
	}

	node = xmlNewChild (root, NULL, "accounts", NULL);
	for (tmp = kf_preferences_accounts; tmp; tmp = tmp->next) {
		xmlNodePtr child;
		
		acc = tmp->data;
		
		child = xmlNewChild (node, NULL, "account", NULL);
		xmlNewProp (child, "name", acc->name);
		xmlNewProp (child, "username", acc->uname);
		xmlNewProp (child, "server", acc->server);
		if (acc->save_password)
			xmlNewProp (child, "password", acc->pass);

		if (acc->port != -1) {
			gchar *text;

			text = g_strdup_printf ("%d", acc->port);
			xmlNewProp (child, "port", text);
			g_free (text);
		}
		if (acc->resource)
			xmlNewProp (child, "resource", acc->resource);
		
		if (acc->priority > -128 && acc->priority < 128) {
			gchar *text;

			text = g_strdup_printf ("%d", acc->priority);
			xmlNewProp (child, "priority", text);
			g_free (text);
		}

		if (acc->proxy_use) {
			xmlNodePtr proxy;
			gchar port[6];

			proxy = xmlNewChild (child, NULL, "proxy", NULL);
			xmlNewTextChild (proxy, NULL, "enable", acc->proxy_use?"yes":"no");
			xmlNewTextChild (proxy, NULL, "server", acc->proxy_server);

			g_snprintf (port, 6, "%d", acc->proxy_port);
			
			xmlNewTextChild (proxy, NULL, "port", port);
			if (acc->proxy_user)
				xmlNewTextChild (proxy, NULL, "user", acc->proxy_user);
			if (acc->proxy_pass)
				xmlNewTextChild (proxy, NULL, "password", acc->proxy_pass);
		}

		xmlNewProp (child, "secure", acc->secure?"yes":"no");
		xmlNewProp (child, "save_passwd", acc->save_password?"yes":"no");
		
				
	}

	node = xmlNewChild (root, NULL, "statuses", NULL);
	for (tmp = kf_preferences_statuses; tmp; tmp=tmp->next) {
		gchar **status = tmp->data;
		xmlNodePtr child;

		child = xmlNewTextChild (node, NULL, "status", status[1]);
		xmlNewProp (child, "name", status[0]);
	}
	
	node = xmlNewChild (root, NULL, "blacklist", NULL);
	for (tmps = kf_preferences_blocked_jids; tmps; tmps=tmps->next) {
		xmlNewTextChild (node, NULL, "item", tmps->data);
	}


	xmlSaveFormatFileEnc (filename, doc, "UTF-8", 1);
	xmlFreeDoc (doc);
	xmlCleanupParser ();
}
static void kf_preferences_load_accounts (xmlDocPtr doc, xmlNodePtr node) {
	
	while (node) {
		xmlChar *name, *uname, *serv, *pass, *port, *res, *priority, *secure, *save_passwd;
		xmlNodePtr *proxy;
		
		if (xmlStrcmp (node->name, (const xmlChar *) "account")) {
			node = node->next;
			continue;
		}

		name = xmlGetProp (node, "name");
		uname = xmlGetProp (node, "username");
		serv = xmlGetProp (node, "server");
		pass = xmlGetProp (node, "password");
		
		port = xmlGetProp (node, "port");
		res = xmlGetProp (node, "resource");
		priority = xmlGetProp (node, "priority");
		secure = xmlGetProp (node, "secure");
		save_passwd = xmlGetProp (node, "save_passwd");
		
		if (name && uname && serv) {
			KfPrefAccount *account;
			xmlNodePtr child;
			
		//	account = g_malloc (sizeof (KfPrefAccount));
			account = kf_pref_account_new (name);
			if (account == NULL)
				return;

//			account->name = g_strdup ((const gchar *) name);
			account->uname = g_strdup ((const gchar *) uname);
			account->server = g_strdup ((const gchar *) serv);
			account->pass = g_strdup ((const gchar *) pass);

			account->port = port?atoi (port):5222;
			account->resource = g_strdup (res);
			account->priority = priority?atoi (priority):5;
			
			if (secure)
				account->secure = (*secure == 'y');

			if (save_passwd)
				account->save_password = (*save_passwd == 'y');

			for (child = node->children; child; child = child->next) {
				if (xmlStrcmp (child->name, (const xmlChar *) "proxy") == 0) {
					xmlNodePtr node;

					for (node = child->children; node; node = node->next) {
						xmlChar *content;

						content = xmlNodeGetContent (node);
						if (xmlStrcmp (node->name, (const xmlChar *) "enable") == 0) {
							account->proxy_use = (content && *content == 'y') ? TRUE:FALSE;
						} else if (xmlStrcmp (node->name, (const xmlChar *) "server") == 0) {
							account->proxy_server = g_strdup (content);
						} else if (xmlStrcmp (node->name, (const xmlChar *) "port") == 0) {
							account->proxy_port = atoi (content);
						} else if (xmlStrcmp (node->name, (const xmlChar *) "user") == 0) {
							account->proxy_user = g_strdup (content);
						} else if (xmlStrcmp (node->name, (const xmlChar *) "password") == 0) {
							account->proxy_pass = g_strdup (content);
						}
						xmlFree (content);
					}
				//	if (*(xmlGetProp (child, "enable")) == 'y')
				//		account->proxy_use = TRUE;
				//	
					
				}
			}

			kf_preferences_accounts = g_list_append
				(kf_preferences_accounts, account);
		}
		xmlFree (name);
		xmlFree (uname);
		xmlFree (serv);
		xmlFree (pass);

		xmlFree (port);
		xmlFree (res);
		xmlFree (secure);
		xmlFree (save_passwd);

		node = node->next;
	}
}

static void kf_preferences_load_statuses (xmlDocPtr doc, xmlNodePtr node) {
	xmlChar *name, *text;
	
	while (node) {
		if (xmlStrcmp (node->name, (const xmlChar *) "status")) {
			node = node->next;
			continue;
		}

		name = xmlGetProp (node, "name");
		if (name) {
			gchar **element;

			element = (gchar **) g_malloc (2 * sizeof (gchar *));
			text = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);

			element[0] = g_strdup (name);
			element[1] = g_strdup (text);

			xmlFree (name);
			xmlFree (text);
			
			kf_preferences_statuses = g_list_append
				(kf_preferences_statuses, element);
		}
		node = node->next;
	}
}

static GSList *kf_preferences_load_list (xmlDocPtr doc, xmlNodePtr node, GSList *list) {
	xmlChar *name, *text;
	
	while (node) {
		if (xmlStrcmp (node->name, (const xmlChar *) "item")) {
			node = node->next;
			continue;
		}

		text = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
		list = g_slist_append (list, g_strdup (text));
		xmlFree (text);
			
		node = node->next;
	}
	return list;
}


/*
const gchar *kf_config_file (const gchar *name) {
	static gchar path[1024];

	g_snprintf (path, 1024, "%s/.kf/%s", g_get_home_dir (), name);
	return path;
}
*/
void kf_preferences_add_status (const gchar *name, const gchar *text) {
	gchar **element;

	element = (gchar **) g_malloc (2 * sizeof (gchar *));

	element[0] = g_strdup (name);
	element[1] = g_strdup (text);

	kf_preferences_statuses = g_list_append
		(kf_preferences_statuses, element);

}

KfPrefAccount *kf_pref_account_new (const gchar *name) {
	KfPrefAccount *acc;

	acc = g_new (KfPrefAccount, 1);
	acc->name= g_strdup (name);
	acc->uname = NULL;
	acc->server = NULL;
	acc->pass = NULL;
	acc->save_password = FALSE;

	acc->resource = NULL;
	acc->port = 5222;
	acc->secure = FALSE;

	acc->proxy_use = FALSE;
	acc->proxy_server = NULL;
	acc->proxy_port = 8080;
	acc->proxy_user = NULL;
	acc->proxy_pass = NULL;

	return acc;
}
	
void kf_pref_account_set_proxy (KfPrefAccount *acc, gboolean use,
				const gchar *server, gint port,
				const gchar *uname, const gchar *passwd) {
	acc->proxy_use = use;
	
	g_free (acc->proxy_server);
	acc->proxy_server = g_strdup (server);
	
	acc->proxy_port = port;
	
	g_free (acc->proxy_user);
	acc->proxy_user = g_strdup (uname);
	
	g_free (acc->proxy_pass);
	acc->proxy_pass = g_strdup (passwd);
}
